/*
@COPYRIGHT@, Dragon Systems Software Limited. All rights reserved.
http://www.dssw.co.uk
Parts based on example code from Apple Computer.
Build-@VERSION@-@DATE@
*/

/************************************/
/* Widget Structure */
/************************************/

var timerIntervalDefault = 120000; // update every two minutes
var timerInterval = timerIntervalDefault;

var panes = new Array;
panes['front'] = { width:135, height:134, from:'back', closeBoxX:27, closeBoxY:27 };
panes['back'] = { width:321, height:227, from:'front', closeBoxX:15, closeBoxY:12 };
var gResizeToPane = 'none';

var gDoneButton;
var gInfoButton;
var gScrollBar;
var gScrollArea;

/* Standard widget set up to avoid cpu cycles while hidden */
if ( window.widget )
{
	widget.onhide = onhide;
	widget.onshow = onshow;
}	

function onshow()
{
    if ( timerInterval == null )
    {
        timerInterval = setInterval( "presentSchedule();", timerIntervalDefault );
    }
    
	presentSchedule();
}

function onhide()
{
    if ( timerInterval != null )
    {
        clearInterval( timerInterval );
        timerInterval = null;
    }
}

function resizeAfterTransition()
{
	if ( panes[gResizeToPane] && window.widget )
	{
		/* Keep the panes aligned, we crawl upwards slightly */
		var to = panes[gResizeToPane];
		var from = panes[to.from];
		var deltaX = ( from.width / 2 ) - ( to.width / 2 );	
		var deltaY = ( from.height / 2 ) - ( to.height / 2 );	
		widget.resizeAndMoveTo(
			window.screenX + deltaX,
			window.screenY + deltaY,
			to.width, to.height
		);
		widget.setCloseBoxOffset( to.closeBoxX, to.closeBoxY );
		gResizeToPane = 'none';
	}
	else
	{
		/* A regrettable trick to force a redraw */
		window.resizeBy( 1, 1 );
		window.resizeBy( -1, -1 );
		
		/* Keep the scroll area aware of flips - redraw */
		gScrollArea.refresh();
	}
}

function flipPrefs( inTransition, inFrontDisplay, inBackDisplay, inToPane )
{
	var front = document.getElementById( "front" );
    var back = document.getElementById( "back" );

    if ( window.widget )
	{
        widget.prepareForTransition( inTransition );
	}

    front.style.display = inFrontDisplay;
    back.style.display = inBackDisplay; 
	gResizeToPane = inToPane;
	resizeAfterTransition();

    if ( window.widget )
	{
        setTimeout( 'widget.performTransition();', 0 );
	}
}

function showPrefs()
{
	flipPrefs( 'ToBack', 'none', 'block', 'back' );
}

function hidePrefs()
{
	flipPrefs( 'ToFront', 'block', 'none', 'front' );
}

function loaded()
{
	// Set up the buttons
	gDoneButton = new AppleInfoButton(document.getElementById("doneButton"), document.getElementById("back"), "white", "white", hidePrefs);
    gInfoButton = new AppleInfoButton(document.getElementById("infoButton"), document.getElementById("front"), "white", "white", showPrefs);

	// Set up the scrollbar
	gScrollBar = new AppleVerticalScrollbar(document.getElementById("scrollbar"));
	gScrollArea = new AppleScrollArea(document.getElementById("scroller"), gScrollBar);
	gScrollArea.scrollsHorizontally = false;
	gScrollArea.singlepressScrollPixels = 25;
	gScrollArea.focus(); // for key control when first loading in Safari
	
	window.onfocus = function () { gScrollArea.focus(); }
	window.onblur = function () { gScrollArea.blur(); }

	/* Listen for transition completion messages to force a redraw */
	if ( window.widget )
	{
		widget.ontransitioncomplete = resizeAfterTransition;
		widget.setCloseBoxOffset( panes['front'].closeBoxX, panes['front'].closeBoxY );
	}
	
	presentSchedule();
}

/************************************/
/* Widget Content */
/************************************/

var events = new Array;
events['wake'] = {icon:'wake',label:'Wake up' };
events['wakepoweron'] = {icon:'poweron',label:'Power on' };
events['poweron'] = {icon:'poweron',label:'Power on' };
events['sleep'] = {icon:'sleep',label:'Sleep' };
events['shutdown'] = {icon:'shutdown',label:'Shut down' };

var icon_prefix = 'images/icon_';
var icon_suffix = '.png';
var icon_none = icon_prefix + 'none' + icon_suffix;

var applications = new Array;
applications['uk.co.dssw.powermanager'] = 'Power Manager';
applications['Repeating'] = 'Energy Saver';

var application_powermanager = 'uk.co.dssw.powermanager';

function trimWhitespace( inString )
{ 
	return inString.replace( /^\s*/, '' ).replace( /\s*$/, '' ); 
}

/* Schedule Event */
/************************************/

function ScheduleEvent( inKind, inApplication, inDate )
{
	this.kind = trimWhitespace( inKind );
	this.application = trimWhitespace( inApplication );
	
	/* Convert the date from a string to an object */
	var pattern = /^(\d+)-(\d+)-(\d+) (\d+):(\d+)$/;
	var result = pattern.exec( trimWhitespace( inDate ) );
	this.date = new Date( result[1], result[2] - 1, result[3], result[4], result[5] );

	this.pending = ScheduleEvent_pending;
	this.debug = ScheduleEvent_debug;
	this.icon = ScheduleEvent_icon;
	this.applicationLink = ScheduleEvent_applicationLink;
}

function ScheduleEvent_pending()
{
	var now = new Date;
	return ( now < this.date );
}

function ScheduleEvent_debug()
{
	return this.kind + " " + this.application + this.date;
}

function ScheduleEvent_icon()
{
	if ( events[this.kind] )
	{
		return icon_prefix + events[this.kind].icon + icon_suffix;
	}
	else
	{
		alert( 'Unknown event kind: ' + this.kind );
		return icon_none;
	}
}

function ScheduleEvent_applicationLink()
{
	if ( applications[this.application] )
	{
		return applications[this.application];
	}
	else
	{
		/* Special case DssW Power Manager */
		if (this.application.indexOf(application_powermanager) != -1)
		{
			return applications[application_powermanager];
		}
	}
	else
	{
		return this.application;
	}
}

/* Schedule Presentation */
/************************************/

function getSchedule()
{
	/* Return an array of scheduled power events */
	var schedule = new Array;
	
	/* Read in the schedule from our external plug in */
	if ( EnergySaverPlugin )
	{
		var scheduleBody = EnergySaverPlugin.getQueue();
		
		/* Nice easy format to parse; comma separated with quotes */
		/* Columns: type, application, date YYYY-MM-DD HH:SS */
		var pattern = /^(.+),(.+),(.+)$/gm;
		var result;
		while ( ( result = pattern.exec( scheduleBody ) ) != null )
		{
			var powerEvent = new ScheduleEvent( result[1], result[2], result[3] );
			
			/* Filter events in the past */
			if ( powerEvent.pending() )
			{
				schedule.push( powerEvent );
			}
		}
	}  
	else
	{
		alert("Widget EnergySaver plugin not loaded.");
	}
	
	return schedule;
}

function presentSchedule()
{
	var schedule = getSchedule();
	
	presentFrontWithScheduleEvent( schedule[0] );
	presentBackSchedule( schedule );
}

function presentFrontWithScheduleEvent( inEvent )
{
	var frontIcon = document.getElementById( "icon" );
	var frontTime = document.getElementById( "time" );
    var frontDate = document.getElementById( "date" );

	if ( inEvent )
	{
		frontIcon.src = inEvent.icon();
		frontTime.innerHTML = inEvent.date.toLocaleTimeString( 'short' );
		frontDate.innerHTML = inEvent.date.toLocaleDateString( 'medium' );
	}
	else
	{
		frontIcon.src = icon_none;
		frontTime.innerHTML = 'Nothing';
		frontDate.innerHTML = 'scheduled';
	}
}

function presentBackSchedule( inSchedule )
{
	if ( inSchedule )
	{
		var firstIcon = document.getElementById( "nexticon" );
		var firstDescription = document.getElementById( "nextdescription" );
		var firstApplication = document.getElementById( "nextapplication" );
		var schedule = document.getElementById( "schedule" );
		var summary = document.getElementById( "summary" );
		var eventCount = document.getElementById( "event_count" );
		var emptyInfo = document.getElementById( "empty" );
		
		/* Present the next event */
		if ( inSchedule.length > 0 )
		{
			var event = inSchedule[0];
		
			firstIcon.src = event.icon();
			firstDescription.innerHTML = events[event.kind].label +
				' at ' +
				event.date.toLocaleTimeString( 'short' ) +
				', ' +
				event.date.toLocaleDateString( 'medium' );
			
			firstApplication.innerHTML = event.applicationLink();
	
			/* Build an HTML list of events */
			var eventList = '<ol id="events" start="2">';
	
			for ( var i = 1; i < inSchedule.length; ++i )
			{
				var event = inSchedule[i];
				
				eventList += '<li><div class="event">';
				eventList += '<div class="action">' + events[event.kind].label + ' at ';
				eventList += event.date.toLocaleTimeString( 'short' ) + ', ' + event.date.toLocaleDateString( 'medium' ) + '</div>';
				eventList += '<div class="scheduledby">Scheduled by <span class="application">' + inSchedule[i].applicationLink() + '</span></div>';
				eventList += '</div></li>';
			}
			
			eventList += '</ol>';
			schedule.innerHTML = eventList;
			
			/* Finally, adjust the summary */
			if ( inSchedule.length == 1 )
			{
				eventCount.innerHTML = inSchedule.length + ' event';
			}
			else
			{
				eventCount.innerHTML = inSchedule.length + ' events';
			}
			
			schedule.style.display = 'block';
			summary.style.display = 'block';
			emptyInfo.style.display = 'none';
			
		}
		else
		{
			firstIcon.src = icon_none;
			firstApplication.innerHTML = '';
			firstDescription.innerHTML = 'No events scheduled.';
			schedule.innerHTML = '';

			schedule.style.display = 'none';
			summary.style.display = 'none';
			emptyInfo.style.display = 'block';
		}
		
		// Adjust the scroll area: resize, reposition the scrollthumb
		gScrollArea.refresh();
	}

}

/************************************/
/* Widget Links */
/************************************/

function openURL( inURL )
{
	if ( window.widget )
	{
		widget.openURL( inURL );
	}
}

function visitDssW()
{
	openURL( 'http://www.dssw.co.uk/' );
}

function visitPowerManager()
{
	openURL( 'http://www.dssw.co.uk/powermanager/?eswidget' );
}

function openEnergySaver()
{
	openURL( 'file:///System/Library/PreferencePanes/EnergySaver.prefPane' );
}